#!/usr/bin/python

import actions
import yaml
import os

from charmhelpers.core import services

from charmhelpers.core.hookenv import (
    config,
    unit_get,
)


def cooked_config():
    conf = config()
    conf['yaml'] = {}

    unit = os.environ.get('JUJU_UNIT_NAME').split('/')[-1]
    conf['yaml']['servername'] = "{}.{}".format(unit, conf['servername'])

    # ssl_ports requires both ssl_key and ssl_certificiate
    if not conf.get('ssl_key') or not conf.get('ssl_certificate'):
        conf['ssl_ports'] = '[]'

    # cook the various yaml formatted strings into their true form
    for key in 'ports', 'ssl_ports', 'ssl_cipher_suite', 'connect_list', \
               'class_list', 'type_list', 'oper_list', 'ldapauth_whitelist', \
               'ldapoper_list':
        val = yaml.safe_load(conf[key])
        if not val:
            val = []
        conf['yaml'][key] = val
    val = yaml.safe_load(conf['peer_ports'])
    if not val:
        val = [['0.0.0.0', 0]]
    conf['yaml']['peer_ports'] = val
    return conf


def IRC_ports_list():
    cook = cooked_config()['yaml']
    addr_ports = cook.get('ports', []) + cook.get('ssl_ports', []) + cook.get('peer_ports', [])
    ret = []
    for addr, port in addr_ports:
        if port:
            ret.append(port)
    return ret


class IRCdRelation(services.RelationContext):
    name = 'ircd'
    inteface = 'inspircd'

    def provide_data(self):
        ret = {}
        conf = config()
        cooked = cooked_config()['yaml']
        ret['ssl_ports'] = conf['ssl_ports']
        if len(cooked['ssl_ports']):
            ret['ssl_port'] = cooked['ssl_ports'][0][1]
        else:
            ret['ssl_port'] = ''
        ret['ports'] = conf['ports']
        if len(cooked['ports']):
            ret['port'] = cooked['ports'][0][1]
        else:
            ret['port'] = ''
        ret['host'] = unit_get('private-address')
        ret['ca_certificate'] = conf['ca_certificate']
        ret['servername'] = conf['servername']
        return ret


class OptionalRelationContext(services.RelationContext):
    required_keys = []

    def is_ready(self):
        return True

    def _is_ready(self, unit_data):
        return True

    def service_available(service_name):
        # not a real service
        return False


class OptionalIRCdPeerRelation(OptionalRelationContext):
    name = 'inspircd'
    inteface = 'inspircd'
    # required_keys = ['servername', 'port', 'host', 'password']

    def provide_data(self):
        ret = {}
        conf = config()
        cooked = cooked_config()['yaml']

        peer_ports = cooked.get('peer_ports', None)
        if not peer_ports:
            peer_ports = [['0.0.0.0', 0]]

        peering = peer_ports[0]       # [IP, port]
        ret['port'] = peering[1]
        if peering[0] == '0.0.0.0':
            ret['host'] = unit_get('private-address')
        else:
            ret['host'] = peering[0]
        ret['servername'] = cooked['servername']
        ret['password'] = conf['password']
        return ret


class OptionalIRCdServicesRelation(OptionalRelationContext):
    name = 'ircdservices'
    interface = 'services'

    # required_keys = ['servername', 'host', 'port', 'password']

    def provide_data(self):
        ret = {}
        conf = config()
        cooked = cooked_config()['yaml']
        ret['protocol'] = 'inspircd20'
        peer_ports = cooked.get('peer_ports', None)
        if not peer_ports:
            peer_ports = [['0.0.0.0', 0]]

        peering = peer_ports[0]       # [IP, port]
        ret['port'] = peering[1]
        if peering[0] == '0.0.0.0':
            ret['host'] = unit_get('private-address')
        else:
            ret['host'] = peering[0]
        ret['servername'] = cooked['servername']
        ret['password'] = conf['password']
        ret['networkname'] = conf['networkname']
        ret['ssl'] = 'no'
        return ret


service_definitions = [
    {
        'service': 'inspircd',
        'ports': IRC_ports_list(),
        'required_data': [cooked_config(), OptionalIRCdPeerRelation(), OptionalIRCdServicesRelation()],
        'data_ready': [
            actions.install_package,
            services.render_template(source="inspircd.conf.tmpl",
                                     target="/etc/inspircd/inspircd.conf"),
            services.render_template(source="default.tmpl",
                                     target="/etc/default/inspircd"),
            services.render_template(source="logrotate.tmpl",
                                     target="/etc/logrotate.d/inspircd"),
            services.render_template(source="nagios.tmpl",
                                     target="/usr/lib/nagios/plugins/check_ircd_ssl",
                                     perms=0755),
            actions.enable_apparmor_profile,
        ],
        'provided_data': [IRCdRelation(), OptionalIRCdPeerRelation(), OptionalIRCdServicesRelation()],
        'start': [actions.inspircd_reload, services.open_ports],
    },
]
